#!/usr/bin/env perl
# Usage: $0 host.fqdn.dreamhost.com tld
use strict;
use 5.010;
use feature qw(state);

use Time::HiRes;
use Net::EPP::Client;
use Net::EPP::Frame::Hello;


########
# Useful variables.

# Nagios standard exit codes.
my $EXIT_OK       = 0;
my $EXIT_WARN     = 1;
my $EXIT_CRITICAL = 2;
my $EXIT_UNKNOWN  = 3;

# Timeout in seconds. If it takes longer than this, we have issues.
my $CHECK_TIMEOUT = 15,

# If it didn't hit CHECK_TIMEOUT, but took longer than WARN_AFTER to finish, we
# should probably flag it as being way too damn slow. To disable this warning,
# set this to 0.
my $WARN_AFTER    = 5,

# The ports the proxies run on.
my %PROXY_PORTS = (
  comnet => 6661,
  info   => 6662,
  org    => 6663,
);


########
# Read in command line options.
my ($host, $tld) = @ARGV;

unless ($host && $tld) {
  print {*STDERR} "Usage: $0 <host> <tld>!\n";
  exit $EXIT_UNKNOWN;
}

# Normalize this silly edge case.
if ($tld eq 'com' || $tld eq 'net') {
  $tld = 'comnet';
}

my $port = $PROXY_PORTS{$tld};
if (!$port) {
  print {*STDERR} "Unknown TLD ($tld): Expected one of com, net, comnet, org, info\n";
  exit $EXIT_UNKNOWN;
}

#######
# Helper so we can output more useful info when we hit a SLOW warning.
# Returns the overall time since we started.
my $timing_log;
sub mark_time {
  my ($note) = @_;
  my $now = Time::HiRes::time;

  state ($start, $last);

  if ($note eq 'start') {
    $start = $last = $now;
    $timing_log = "Started at $start\n";
    return 0;
  }

  my $delta_l = $now - $last;
  my $delta_s = $now - $start;
  $last = $now;

  $timing_log .= "Event $note:\n";
  $timing_log .= sprintf "    delta since last:  %0.2f\n", $delta_l;
  $timing_log .= sprintf "    delta since start: %0.2f\n", $delta_s;

  return $delta_s;
}

########
# Do the main bit of magic!
my $result = 'generic failure';
mark_time('start');
eval {
  local $SIG{ALRM} = sub { die "timed out (after $CHECK_TIMEOUT sec)" };
  alarm $CHECK_TIMEOUT;
  epp_ping_registry($host, $port);
  $result = 'ok'
};
alarm 0;
if ($@ && $result ne 'ok') {
  $result = $@;
}
my $duration = mark_time('finished');


########
# Handle results
if ($result ne 'ok') {
  print $result, "\n";
  exit $EXIT_CRITICAL;
}

if ($WARN_AFTER > 0 && $duration >= $WARN_AFTER) {
  print "SLOW! Took $duration seconds to run (threshold was $WARN_AFTER)\n\n$timing_log\n";
  exit $EXIT_WARN;
}

print "OK!\n";
exit $EXIT_OK;


########
# The meat of the check -- connect and send a hello frame.
# On initial connect, the Proxy sends back a cached Greeting.
# On receiving a Hello, the Remote Server sends back a fresh Greeting.
sub epp_ping_registry {
  my ($host, $port) = @_;

  my $Client = Net::EPP::Client->new(
    host   => $host,
    port   => $port,
    frames => 1,
  );

  # Connect
  mark_time('Connecting...');
  my $CachedGreeting = eval { $Client->connect };
  if ($@) {
    die "Connection failed: $@";
  }
  mark_time('Connected');
  epp_validate_greeting('proxy', $CachedGreeting);

  # And send a Hello frame.
  my $Hello = Net::EPP::Frame::Hello->new;
  mark_time('Sending Hello frame.');
  my $FreshGreeting = eval { $Client->request($Hello) };
  if ($@) {
    die "Failed to send Hello packet: $@";
  }
  mark_time('Received response to Hello.');
  epp_validate_greeting('registry', $FreshGreeting);

  return;
}

# Attempt to do the crudest possible validation on the frame.
sub epp_validate_greeting {
  my ($source, $Greeting) = @_;

  # Is it a standard greeting? If so, we're good!
  if ($Greeting->getNode('greeting')) {
    return;
  }

  # Below here be failures. With any failure of this magnitude, we should
  # output the entire frame for later debugging.
  my $frame = $Greeting->toString();
  my $error = 'unexpected/unrecognized frame';

  # If it's a standard EPP error frame, yield a decent error.
  if (my $Result = $Greeting->getNode('result')) {
    if (my ($Msg) = $Result->getChildrenByTagName('msg')) {
      my $code = $Result->getAttribute('code');
      my $msg  = $Msg->textContent;

      $error = "EPP Error $code: $msg";
    }
  }

  die "Expected greeting from $source; got $error.\nFrame was:\n$frame\n";
}
